require_relative '../../support/helper'

require 'tango/resource/buffer'

class TestBuffer < Test::Unit::TestCase
      
  context "a buffer" do
      
    setup do
      @buffer_size = 10
      @buffer = Tango::Resource::Buffer.new( @buffer_size )
    end
    
    should "throw exception when try register type without callback object with unregistered type" do
      assert_raise ArgumentError do
        @buffer.register( :foo )
      end
    end
    
    should "throw exception when try to fill object with unregistered type" do
      assert_raise ArgumentError do
        @buffer.fill( :foo, Object.new )
      end
    end
    
    should "give ability to register new type of resource" do
      @buffer.register( :foo ) do; nil; end
      @buffer.fill( :foo, Object.new )
    end
    
    should "trigger release callback when buffer size exceeded" do
      
      counter = 0
      
      @buffer.register( :foo ) do
        counter += 1
      end
      
      ( @buffer_size * 5 ).times do 
        @buffer.fill( :foo, Object.new )
      end # Full buffer 5 times
      
      assert_equal 5, counter
      
    end
  
  end
  
end