require_relative '../../support/helper'
require_relative '../../support/lib/simple_buffer'

require 'tango/resource/cache'

class TestCache < Test::Unit::TestCase
      
  context "a cache" do
      
    setup do
      @buffer = SimpleBuffer.new
      @cache = Tango::Resource::Cache.new( @buffer )
      @foo_resource = stub( cache_key: "foo" )
    end
    
    should "throw exception when trying to load unregistered type" do
      assert_raise ArgumentError do
        @cache.load( :foo, "bar" )
      end
    end
    
    should "throw exception when trying to load yet not cached resource without callback" do
      
      @cache.register( :foo ) do
        nil
      end
            
      assert_raise ArgumentError do
        @cache.load( :foo, @foo_resource )
      end
      
    end
    
    should "be able to store recourse" do
      
      @cache.register( :foo ) do
        nil
      end
      
      @cache.set( :foo, @foo_resource )
      assert_equal @foo_resource, @cache.get( :foo, @foo_resource )
      
    end
    
    should "be able to load already stored recourse" do
      
      @cache.register( :foo ) do
        nil
      end
      
      @cache.set( :foo, @foo_resource )
      assert_equal @foo_resource, @cache.load( :foo, @foo_resource )
      
    end
    
    should "load yet not cached value using callback" do
      
      @cache.register( :foo ) do
        nil
      end
      
      loaded_resource = @cache.load( :foo, @foo_resource ) do
        @foo_resource
      end
      
      assert_equal @foo_resource, loaded_resource
      assert_equal @foo_resource, @cache.load( :foo, @foo_resource )
      
    end
    
    should "also register new types with buffer" do
    
      10.times do |i|
        @cache.register( i ) do
          nil
        end
      end
      
      assert_equal 10, @buffer.register_counter
      
    end
    
    should "fill buffer with loaded resources" do
      
      @cache.register( :foo ) do
        nil
      end
      
      2.times do |i|
        resouce = stub( cache_key: i )
        loaded_resource = @cache.load( :foo, resouce ) do
          resouce
        end
      end
      
      assert_equal 2, @buffer.fill_couter
      
    end
    
    should "fill buffer only once with same resource" do
      
      @cache.register( :foo ) do
        nil
      end
      
      2.times do |i|
        loaded_resource = @cache.load( :foo, @foo_resource ) do
          @foo_resource
        end
      end
      
      assert_equal 1, @buffer.fill_couter
      
    end
  
  end
  
end