require_relative '../support/helper'
require 'tango/link_stack'

class TestLinkStack < Test::Unit::TestCase

  context "a link stack" do
    
    setup do 
      @stack = Tango::LinkStack.new( 'http://example.com/data?xml' )
    end
    
    should "raise error when initialized with incorrect URL" do
      assert_raise ArgumentError do
        Tango::LinkStack.new( 'ImNotA/Link' )
      end
    end
    
    should "know host of base link" do
      assert_equal 'http://example.com:80', @stack.host
    end
    
    should "contain initial link path" do
      assert @stack.has_links?
      assert_equal ['/data?xml'], @stack.links 
      assert_equal '/data?xml', @stack.shift
      assert ! @stack.has_links?
    end
    
    should "store appended links" do
      
      @stack.shift # shift initial path
      
      @stack.append '/data/bids'
      @stack.append [ '/data/bids/1', '/data/bids/2' ]
      assert_equal 3, @stack.links.count
      
      assert_equal '/data/bids', @stack.shift 
      assert_equal '/data/bids/1', @stack.shift
      assert_equal '/data/bids/2', @stack.shift
      
      assert ! @stack.has_links?
      
    end
      
  end
end